
import pylab
import os

from variables import *

log10_for_variance_treshhold = 100
log10_for_variance_cutoff    = 1e-2
colors = ['b', 'g', 'r', 'c', 'm', 'y', 'burlywood', 'chartreuse', 'k']

import matplotlib as mpl
mpl.rcParams['font.size'] = 18
mpl.rcParams['legend.fontsize'] = 16

class Run (object):
    def __init__(self, name=None, path=''):
        if name:
            self.outputf = path + 'output_'+name+'.dat'
            self.varf = path + 'var_output_'+name+'.dat'
            self.timesf = path + 'times_'+name+'.py'
            self.configf = path + 'config_'+name+'.py'
            self.runtimef = path + 'runtime_'+name+'.py'
            self.movief = path + 'movie_'+name
            self.name = name
        else:
            self.outputf = path + 'output.dat'
            self.varf = path + 'var_output.dat'
            self.timesf = path + 'times.py'
            self.configf = path + 'config.py'
            self.runtimef = path + 'runtime.py'
            self.movief = path + 'movie'
            self.name = path + 'run'
        times = {}
        execfile(self.timesf, globals(), times)
        self.times = times['time']
        config = {}
        execfile(self.configf, globals(), config)
        self.opts = config['opts']
        self.vars = config['used']
        if 'info' in config:
          self.info = config['info']
        else:
          self.info = None
        self.runtime = {}
        execfile(self.runtimef, globals(), self.runtime)
        self.i = Config(self, int)
        self.f = Config(self, float)
        self.nx = self.i.NX
        self.ny = self.i.NY
        self.nz = self.i.NZ
        
        # extracting the data (infolines) to be displayed on the plots
        self.info_opts = self.opts
        self.info_vars = config['vars']
        if 'RUNTIME_FILE' in self.info_vars: del self.info_vars ['RUNTIME_FILE']
        if 'LOG_FILE'     in self.info_vars: del self.info_vars ['LOG_FILE']
        if 'TIME_FILE'    in self.info_vars: del self.info_vars ['TIME_FILE']
        if 'DATA_FILE'    in self.info_vars: del self.info_vars ['DATA_FILE']
        if 'DATA_PATH'    in self.info_vars: del self.info_vars ['DATA_PATH']
        
        # detecting number of variables from equation type
        if 'equation' in self.opts:
          self.NVARS = get_NVARS (self.opts['equation'])
        else:
          self.NVARS = 16
    
    def var(self, v, ts, case="mean"):
        if ts > len(self.times):
            raise Exception ("Timestep out of bounds")
        if case == "mean":
            f = open(self.outputf, 'rb')
        else: # case="variance"
            f = open(self.varf, 'rb')
        n = self.nx*self.ny*self.nz
        f.seek(8*n*(v.i + self.NVARS*ts))
        fd = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        fd.shape = (self.nx, self.ny, self.nz)
        return fd
    
    def showInfo(self, prefix='', adjust=True):
      
      string_opts = ''.join([' %s:%s,' % x for x in sorted(self.info_opts.items())])
      string_vars = ''.join([' %s=%s' % x for x in sorted(self.info_vars.items())])
      
      text  = prefix + 'OPTS:' + string_opts[0:-1] + '\n'
      text += prefix + 'VARS:' + string_vars + '\n'
      
      if 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime']
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      text += prefix + 'INFO: '
      if self.info:
        text += 'cluster: ' + str(self.info['CLUSTER']) + ', '
      text += 'cores: ' + str(self.runtime['nodes']) + ', ' + 'runtime: ' + runtime
      
      if 'mpi_scaling' in self.runtime:
        text += ', ' + 'efficiency: ' + str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'%'
      
      pylab.figtext(0.01, 0.015, text, fontsize='small')
      
      if adjust:
        pylab.subplots_adjust(bottom=0.20)
    
    def generateTexTable(self, save=None):
      
      # config
      
      keys     =  ['L',    'ML',     'grid_size', 'CFL', 'cores', 'runtime']
      captions =  [r'$L$', r'$M_L$', 'grid size', 'CFL', 'cores', 'runtime']
      if 'mpi_scaling' in self.runtime:
        keys     += ['efficiency']
        captions += ['efficiency']
      
      # aggregation of information
      
      opts = {}
      opts.update(self.opts)
      opts.update(self.vars)
      
      grid_size = str(opts['NX'])
      if self.ny != 1:
        grid_size += 'x' + str(opts['NY'])
      if self.nz != 1:
        grid_size += 'x' + str(opts['NZ'])
      opts.update( { 'grid_size' : grid_size } )
      
      opts.update( { 'cores' : str(self.runtime['nodes']) } )
      
      if 'wall_runtime' in self.runtime:
        runtime = self.runtime['wall_runtime']
      else:
        runtime = self.runtime['clock_runtime']
      
      import datetime
      runtime = str(datetime.timedelta(seconds=round(runtime)))
      
      opts.update( { 'runtime' : str(runtime) } )
      
      if 'mpi_scaling' in self.runtime:
        opts.update( { 'efficiency' : str(round(1000 * self.runtime['mpi_scaling']) / 10.0) + r'\%' } )
      
      # TeX code generation
      
      columns =  '|' + 'c|' * len(keys)
      
      text =  '\n'
      text += r'\begin{tabular}{%s}' % columns + '\n'
      
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(captions))[0:-2] % tuple(captions) + r'\\' + '\n'
      text += r'\hline' + '\n'
      text += (r'%s & ' * len(keys))[0:-2] % tuple([opts[key] for key in keys]) + r'\\' + '\n'
      text += r'\hline' + '\n'
      
      text += r'\end{tabular}' + '\n'
      
      # saving
      
      f = open(save[0:-4] + '.tex', 'w')
      f.write(text)
      f.close()
    
    def plotTopography_1d(self, xr, ts):
        b_mean = self.var(sw.B, ts, "mean")[:,0,0]
        
        if int(self.vars['COMPUTE_VARIANCE']) == 0:
          pylab.plot(xr, b_mean, "k-", label="topography")
        else:
          pylab.plot(xr, b_mean, "k-")
        
        if int(self.vars['COMPUTE_VARIANCE']) == 1:
          b_var = self.var(sw.B, ts, "variance")[:,0,0]
          b_std = numpy.sqrt(b_var)
          pylab.plot(xr, b_mean + b_std, "k--")
        
        ymin, ymax = pylab.ylim()
        pylab.fill_between(xr, b_mean, ymin, alpha=0.5, color='black')
    
    def plotTopography_2d(self, extent, ts, case, N=4):
        b = self.var(sw.B, ts, case)[:,:,0]
        cp = pylab.contour(numpy.transpose(b), N, extent=extent, colors='black')
    
    def plot(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='best', cut_slice=None, contour=False, log_var=False, poster=False):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.plot_1d(v, ts, extent, frame, infolines, save, topography, loc)
        else:
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography=False, cut_slice=cut_slice, contour=contour, log_var=log_var, poster=poster)
    
    def plot0(self, v, frame=False, ts=None, extent=None, infolines=True, save=None, topography=True, loc='best', cut_slice=None, contour=False):
        if not int(self.vars['SAVE0']):
            raise Exception ("Snapshot at t=0 is not available! Use SAVE0=1 when compiling.")
        
        if ts is None: ts = len(self.times)-1
        
        if self.ny == 1:
            if not frame:
              if infolines:
                pylab.figure(figsize=(2*8,6))
              else:
                pylab.figure(figsize=(2*8,5))
            pylab.subplot(121)
            self.plot_1d(v, 0, extent, True, False, topography=topography, loc=loc)
            pylab.subplot(122)
            self.plot_1d(v, ts, extent, frame, infolines, save, topography=topography, loc=loc)
        else:
            self.subplot_2d(v, 0, extent,  extent, frame, infolines, save, topography, cut_slice, contour=contour)
            self.subplot_2d(v, ts, extent, extent, frame, infolines, save, topography, cut_slice, contour=contour)
    
    def subplot(self, v, frame=False, ts=None, extent_mean=None, extent_var=None, infolines=True, save=None, topography=True, movie=False, loc='best', cut_slice=None, contour=False, log_var=False):
        if ts is None: ts = len(self.times)-1
        if self.ny == 1:
            self.subplot_1d(v, ts, extent_mean, extent_var, frame, infolines, save, movie, loc=loc, topography=topography)
        else:
            self.subplot_2d(v, ts, extent_mean, extent_var, frame, infolines, save, topography=False, cut_slice=cut_slice, movie=movie, contour=contour, log_var=log_var)

    def plot_1d(self, v, ts, extent=None, frame=False, infolines=True, save=None, x=None, y=None, z=None, topography=True, loc='best'):
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        mean = self.var(v, ts, "mean")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          mean += self.var(sw.B, ts, "mean")
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          mean_label = "mean"
        else:
          mean_label = "solution"
        
        pylab.plot(xr, mean[:,0,0], label=mean_label)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          var = self.var(v, ts, "variance")
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            var -= self.var(sw.B, ts, "variance")
          std = numpy.sqrt(numpy.fabs(var));
          upper_std = mean + std;
          lower_std = mean - std;
          pylab.plot(xr, upper_std[:,0,0], "--", label="mean + std")
          pylab.plot(xr, lower_std[:,0,0], "--", label="mean - std")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(xr, ts)
        
        if extent:
            pylab.ylim(*extent)
        pylab.title(v.name + ' at t=%.2f' % self.times[ts])
        
        pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.96)
        pylab.subplots_adjust(left=0.06)
        # bugfix in pylab for topography plotting
        if not 'equation' in self.opts or not self.opts['equation'] == 'sw' or not topography or not v == sw.H:
          pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if save:
            pylab.savefig(save)
            self.generateTexTable(save)
        if not frame:
            pylab.show()
    
    def subplot_1d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, movie=False, x=None, y=None, z=None, topography=True, loc='best'):
        
        if movie:
          pylab.figure(figsize=(2*8,6))
        
        xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        pylab.plot(xr, d[:,0,0], label=self.name)
        pylab.title('mean' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and v == sw.H:
          self.plotTopography_1d(xr, ts)
        
        pylab.legend(loc=loc)
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          pylab.plot(xr, d[:,0,0], label=self.name)
          if extent_var:
              pylab.ylim(*extent_var)
          if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          else:
            pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f' % self.times[ts])          
          pylab.legend(loc=loc)
        
        pylab.subplots_adjust(right=0.98)
        pylab.subplots_adjust(left=0.03)
        pylab.subplots_adjust(top=0.94)
        
        if infolines: self.showInfo()
        
        if save:
           pylab.savefig(save)
           if not movie: self.generateTexTable(save)
        if not frame and not movie:
            pylab.show()
    
    def subplot_2d(self, v, ts, extent_mean=None, extent_var=None, frame=False, infolines=True, save=None, topography=False, cut_slice=None, movie=False, log_var=False, contour=False, poster=False):
        
        if cut_slice:
          self.subplot_2d_slice (v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie, log_var=log_var)
          return
        
        if infolines:
          pylab.figure(figsize=(14,6))
        else:
          pylab.figure(figsize=(14,5))
        
        # mean
        s = {}
        if extent_mean:
          s['vmin'] = extent_mean[0]
          s['vmax'] = extent_mean[1]
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        
        if contour:
          pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
        else:
          pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
        pylab.title('mean' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
        if poster: pylab.axis('off')
        
        # MUST be before all 'pylab.colorbar()' or otherwise plot is distorted (any idea why?)
        if infolines:
          self.showInfo()
          pylab.subplots_adjust(right=0.96)
          pylab.subplots_adjust(left=0.00)
          pylab.subplots_adjust(top=0.94)
          pylab.subplots_adjust(wspace=0.05)
        else:
          pylab.subplots_adjust(right=0.96)
          pylab.subplots_adjust(left=0.00)
          pylab.subplots_adjust(top=0.92)
          pylab.subplots_adjust(wspace=0.05)
        
        pylab.colorbar()
        if 'equation' in self.opts and self.opts['equation'] == 'sw':
          pylab.winter()
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and not contour:
          self.plotTopography_2d((0,self.f.MAXX,0,self.f.MAXY), ts, "mean")
        
        # variance
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          s = {}
          if extent_var:
            if log_var:
              s['vmin'] = numpy.log10( numpy.max( log10_for_variance_cutoff, extent_var[0] ) )
              s['vmax'] = numpy.log10( extent_var[1] )
            else:
              s['vmin'] = extent_var[0]
              s['vmax'] = extent_var[1]
          
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          
          pylab.subplot(122)
          
          '''
          if extent_var:
            if contour:
              pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          '''
          
          if log_var:
            var_cutoff = numpy.transpose(d[:,:,0])
            var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
            if contour:
              pylab.contour(numpy.log10(var_cutoff), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.log10(var_cutoff), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          else:
            if contour:
              pylab.contour(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            else:
              pylab.imshow(numpy.transpose(d[:,:,0]), origin='lower', extent=(0,self.f.MAXX,0,self.f.MAXY), **s)
            pylab.title('variance' + ' of ' + v.name + ' at t=%.2f' % self.times[ts])
          
          pylab.colorbar()
          if 'equation' in self.opts and self.opts['equation'] == 'sw':
            pylab.winter()
          
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and topography and not contour:
            self.plotTopography_2d((0,self.f.MAXX,0,self.f.MAXY), ts, "variance")
          
          if poster: pylab.axis('off')
        
        if save:
          pylab.savefig(save)
          if not movie: self.generateTexTable(save)
        if not frame and not movie:
          pylab.show()
    
    def subplot_2d_slice(self, v, ts, extent_mean, extent_var, frame, infolines, save, cut_slice, movie, log_var=False):
        
        if infolines:
          pylab.figure(figsize=(14,6))
        else:
          pylab.figure(figsize=(14,5))
        
        xr = []
        slice_at = 0
        if cut_slice['dir'] == 'X':
          xr = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
          slice_at = round(float(cut_slice['pos']) / self.f.MAXX * self.nx)
        else:
          xr = numpy.arange(self.ny)*(self.f.MAXY/self.ny)+0.5*self.f.MAXY/self.ny
          slice_at = round(float(cut_slice['pos']) / self.f.MAXY * self.ny)
        
        d = self.var(v, ts, 'mean')
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          d += self.var(sw.B, ts, "mean")
        
        pylab.subplot(121)
        if cut_slice['dir'] == 'X':
          pylab.plot(xr, d[slice_at,:,0], label=self.name)
          pylab.title('mean' + ' of ' + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        else:
          pylab.plot(xr, d[:,slice_at,0], label=self.name)
          pylab.title('mean' + ' of ' + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
        pylab.legend(loc='upper right')
        
        if extent_mean:
            pylab.ylim(*extent_mean)
        
        if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
          d = self.var(v, ts, 'variance')
          if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
            d = numpy.fabs(d - self.var(sw.B, ts, "variance"))
          pylab.subplot(122)
          if cut_slice['dir'] == 'X':
            if not log_var:
              pylab.plot(xr, d[slice_at,:,0], label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('variance' + ' of ' + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              var_cutoff = d[slice_at,:,0]
              var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
              pylab.plot(xr, numpy.log10(var_cutoff), label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('log10(kurtosis)' + ' of ' + v.name + ' t=%.2f, x=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          else:
            if not log_var:
              pylab.plot(xr, d[:,slice_at,0], label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('variance' + ' of ' + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('4th centered moment' + ' of ' + v.name + ' t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
            else:
              var_cutoff = d[:,slice_at,0]
              var_cutoff = numpy.where(var_cutoff < log10_for_variance_cutoff, log10_for_variance_cutoff, var_cutoff)
              pylab.plot(xr, numpy.log10(var_cutoff), label=self.name)
              if not 'COMPUTE_KURTOSIS' in self.vars or int(self.vars['COMPUTE_KURTOSIS']) == 0:
                pylab.title('log10(variance)' + ' of ' + v.name + ' at t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
              else:
                pylab.title('log10(kurtosis)' + ' of ' + v.name + ' t=%.2f, y=%.2f' % (self.times[ts], float(cut_slice['pos'])))
          if extent_var:
              pylab.ylim(*extent_var)
          pylab.legend(loc='upper right')
        
        pylab.subplots_adjust(right=0.97)
        pylab.subplots_adjust(left=0.07)
        pylab.subplots_adjust(top=0.94)
        pylab.subplots_adjust(wspace=0.30)
        
        if infolines: self.showInfo()
        
        if save:
            pylab.savefig(save)
            if not movie: self.generateTexTable(save)
        if not frame and not movie:
            pylab.show()
    
    def getExtents (self, v, ts, margin=0.0):
      mean = self.var(v, ts, "mean")
      if 'equation' in self.opts and self.opts['equation'] == 'sw':
        mean += self.var(sw.B, ts, "mean")
      
      low  = numpy.min(mean)
      high = numpy.max(mean)
      diff = high - low
      
      low  -= diff * margin
      high += diff * margin
      
      ext_mean = (low, high)
      
      if not 'COMPUTE_VARIANCE' in self.vars or int(self.vars['COMPUTE_VARIANCE']) == 1:
        var  = self.var(v, ts, "variance")
        
        if 'equation' in self.opts and self.opts['equation'] == 'sw' and v == sw.H:
          var = numpy.fabs(var - self.var(sw.B, ts, "variance"))
        
        low  = numpy.min(var)
        high = numpy.max(var)
        diff = high - low
        
        low  -= diff * margin
        high += diff * margin
      
      ext_var = (low, high)
      
      return [ext_mean, ext_var]
    
    def movie(self, v, slowdown=5, infolines=True, last_extent=False, cut_slice=None, topography=False, log_var=False):
        
        margin = 0.1
        
        # for 2d plots, colorbars also indicate both extrema, hence should not be modified
        if self.i.NY != 1:
          margin = 0.0
        
        if last_extent:
          [ext_mean, ext_var] = self.getExtents(v, len(self.times)-1, margin)
        else:
          [ext_mean, ext_var] = self.getExtents(v, 0, margin)
          
          for ts in range(1,len(self.times)):
              
              [ext_mean_tmp, ext_var_tmp] = self.getExtents(v, ts, margin)
              
              ext_mean = (min(ext_mean_tmp[0], ext_mean[0]), max(ext_mean_tmp[1], ext_mean[1]))
              ext_var  = (min(ext_var_tmp [0], ext_var [0]), max(ext_var_tmp [1], ext_var [1]))
        
        for ts in range(len(self.times)):
            self.subplot(v, False, ts, ext_mean, ext_var, infolines, topography=topography, movie=True, cut_slice=cut_slice, log_var=log_var)
            pylab.savefig('movie_%04d.png' % ts)
            pylab.close()
        
        fps = round ( int(self.vars['NSAVES']) / float(self.vars['MAXT']) )
        
        os.system('mencoder "mf://*.png" -mf fps=%d -o %s_%s.mp4 -ovc lavc -lavcopts vcodec=mpeg4:vbitrate=800' % (fps/slowdown, self.movief, v.name))
        #os.system('mencoder "mf://*.png" -mf fps=%d -o %s.avi -ovc lavc -lavcopts vcodec=msmpeg4v2:vbitrate=800' % (fps/slowdown, self.movief))
        
        os.system('rm movie_*.png')
    
    def matlab(self, v, ts=None, path=''):
        if ts is None: ts = len(self.times)-1
        data = {}
        
        for case in ['mean', 'variance']:
          data[case] = self.var(v, ts, case)[:,0,0]
        
        if 'wall_runtime' in self.runtime:
          data['time'] = self.runtime['wall_runtime']
        else:
          data['time'] = self.runtime['clock_runtime']
        
        data['x'] = numpy.arange(self.nx)*(self.f.MAXX/self.nx)+0.5*self.f.MAXX/self.nx
        
        from scipy.io import savemat
        savemat(path + self.name + '_' + v.name + '.mat', data)

def L1(data, refd=None):
    if refd is None:
        n = self.f.MAXX*self.f.MAXY*reduce((lambda x,y:x*y), data.shape)
        return abs(data).sum()/n
    else:
        return abs(data - refd).sum()/abs(refd).sum()

def Linf(data, refd=None):
    if refd is None:
        return numpy.max(abs(data))
    else:
        return numpy.max(abs(data - refd))/numpy.max(abs(refd))

def tableref(r, runref, v):
    fmt="%10.1e"
    scale=1
    t1 = len(r.times)-1
    t2 = len(runref.times)-1
    d = r.var(v, t1)
    refd = runref.var(v, t2)
    zoom = runref.i.NX/r.i.NX
    if runref.i.NY == 1:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,:,:]+refd[1:runref.i.NX:2,:,:])/2
            zoom=zoom/2
    else:
        while zoom>1:
            refd = (refd[0:runref.i.NX-1:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,0:runref.i.NY-1:2,:]+refd[1:runref.i.NX:2,1:runref.i.NY:2,:]+refd[0:runref.i.NX-1:2,1:runref.i.NY:2,:])/4
            zoom=zoom/2
    td = (L1(d, refd)*scale, Linf(d, refd)*scale)
    print ((r"%s & %s & %s \\"% (r.name,fmt,fmt)) % td)

def rebin(d, f):
    nx,ny,nz = d.shape
    if ny == 1:
        assert nz == 1
        return sum(d[i:nx-f+1+i:f,:,:] for i in range(f))/f
    elif nz == 1:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,:] for i in range(f))/(f*f)
    else:
        return sum(d[i:nx-f+1+i:f,i:ny-f+1+i:f,i:nz-f+1+i:f] for i in range(f))/(f*f*f)

def table(runs, v, t=None, fmt="%12.1e", refd=None, reft=None, refr=None, scale=1, dt=0.01):
    if t is None:
        t = max(r.times[-1] for r in runs)
    print (r'       Run &        $L_1$ &   $L_\infty$ \\')
    for r in runs:
        ts = None
        for n,tm in enumerate(r.times):
            if abs(tm - t) < dt:
                ts = n
                break
        if ts is None:
            print (r.name + r' & crashed \\')
            continue
        d = r.var(v, ts)
        if reft is not None:
            refts = None
            for n,tm in enumerate(r.times):
                if tm == reft:
                    refts = n
                    break
            refd = r.var(v, refts)
        elif refr is not None:
            refd = refr.var(v, ts)
            if refr.i.NX != r.i.NX:
                assert (refr.i.NX//r.i.NX) * r.i.NX == refr.i.NX
                refd = rebin(refd, refr.i.NX // r.i.NX)
        td = (L1(d, refd)*scale, Linf(d, refd)*scale)
        print ((r"%10s & %s & %s \\"% (r.name,fmt,fmt)) % td)

def interpolate(v, x, y):
    x0 = int(numpy.floor(x))
    y0 = int(numpy.floor(y))
    x1 = int(numpy.ceil(x))
    y1 = int(numpy.ceil(y))
    if x1 >= v.shape[0]: x1 = v.shape[0]-1
    if y1 >= v.shape[1]: y1 = v.shape[1]-1
    if x0 == x1:
        if y0 == y1:
            return v[x0,y0]
        return v[x0,y0]*(y-y0) + v[x0,y1]*(y1-y)
    elif y0 == y1:
        return v[x0,y0]*(x-x0) + v[x1,y0]*(x1-x)
    return (v[x0,y0]*(x-x0) + v[x1,y0]*(x1-x))*(y-y0) + \
        (v[x0,y1]*(x-x0) + v[x1,y1]*(x1-x))*(y1-y)

def fieldlines(fx, fy, xsc=1.0, ysc=1.0, nlines=20, colour='k'):
    assert fx.shape == fy.shape
    dfy0 = sum(x for x in fy[:,0] if x > 0) / (nlines+1)
    fy0 = 0
    x0 = 0
    for li in range(nlines):
        while fy0 < dfy0:
            if fy[x0,0] > 0: fy0 += fy[x0,0]
            x0 += 1
        fy0 -= dfy0
        x = x0 - fy0/fy[x0-1,0]
        y = 0
        lx = [xsc*x]
        ly = [ysc*y]
        while x >= 0 and y >= 0 and x <= fx.shape[0] and y <= fx.shape[1]:
            dx = interpolate(fx, x, y)
            dy = interpolate(fy, x, y)
            h = numpy.sqrt(dx*dx+dy*dy)
            x += dx/h
            y += dy/h
            lx.append(xsc*x)
            ly.append(ysc*y)
        pylab.plot(lx, ly, colour)

print ("")

try:
  r = Run();
  print (" :: Default run loaded as \"r\".")
except:
  print (" :: Default run not found!")

try:
  mc = Run('MC');
  print (" :: MC run loaded as \"mc\".")
except:
  print (" :: MC run not found!")

try:
  mlmc = Run('MLMC');
  print (" :: MLMC run loaded as \"mlmc\".")
except:
  print (" :: MLMC run not found!")

try:
  mlmcd = Run('MLMCd');
  print (" :: MLMCd run loaded as \"mlmcd\".")
except:
  print (" :: MLMCd run not found!")

try:
  mc2 = Run('MC2');
  print (" :: MC2 run loaded as \"mc2\".")
except:
  print (" :: MC2 run not found!")

try:
  mlmc2 = Run('MLMC2');
  print (" :: MLMC2 run loaded as \"mlmc2\".")
except:
  print (" :: MLMC2 run not found!")

try:
  mlmc2d = Run('MLMC2d');
  print (" :: MLMC2d run loaded as \"mlmc2d\".")
except:
  print (" :: MLMC2d run not found!")

print ("")

def tables (var):
  table([mc, mlmc, mlmcd, mc2, mlmc2, mlmc2d], var, None, "%12.1e", None, None, ref)

# helpers

# path for local saving of figures with descriptive TeX files
IMG_PATH = 'img'

def getFileName (refr, suffix=None, file_suffix='pdf'):

  if not os.path.exists(IMG_PATH):
    os.mkdir(IMG_PATH)
    os.system('svn add ' + IMG_PATH)
  name = os.path.join(IMG_PATH,refr.opts['model'])
  if 'equation' in refr.opts and refr.opts['equation'] == 'sw':
    name += '-' + refr.opts['topography']
  if suffix:
    name += '_' + suffix
  if file_suffix:
    name += '.' + file_suffix
  else:
    name += '.pdf'
  return name

def svnup():
  if not os.path.exists(IMG_PATH):
    os.system('svn up')

def svnadd():
  os.system('svn add -q ' + os.path.join(IMG_PATH, '*'))
